@@ -9,21 +9,24 @@
 //* intact.
 //*********************************************************************
 
-#ifndef Base64_H
-#define Base64_H
+#ifndef TALK_BASE_BASE64_H__
+#define TALK_BASE_BASE64_H__
 
 #include <string>
-using std::string;  // comment if your compiler doesn't use namespaces
+
+namespace talk_base {
 
 class Base64
 {
 public:
-  static string encode(const string & data);
-  static string decode(const string & data);
-  static string encodeFromArray(const char * data, size_t len);
+  static std::string encode(const std::string & data);
+  static std::string decode(const std::string & data);
+  static std::string encodeFromArray(const char * data, size_t len);
 private:
-  static const char cb64[];
-  static const char cd64[];
+  static const std::string Base64::Base64Table;
+  static const std::string::size_type Base64::DecodeTable[];
 };
 
-#endif
+} // namespace talk_base
+
+#endif // TALK_BASE_BASE64_H__
