@@ -25,12 +25,12 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <algorithm>
+#include <cassert>
+
 #include "talk/base/basictypes.h"
 #include "talk/base/bytebuffer.h"
 #include "talk/base/byteorder.h"
-#include <algorithm>
-#include <cassert>
-#include <cstring>
 
 #if defined(_MSC_VER) && _MSC_VER < 1300
 namespace std {
@@ -38,9 +38,9 @@ namespace std {
 }
 #endif
 
-namespace cricket {
+namespace talk_base {
 
-const int DEFAULT_SIZE = 4096;
+static const int DEFAULT_SIZE = 4096;
 
 ByteBuffer::ByteBuffer() {
   start_ = 0;
@@ -66,7 +66,7 @@ ByteBuffer::ByteBuffer(const char* bytes) {
 }
 
 ByteBuffer::~ByteBuffer() {
-  delete [] bytes_;
+  delete bytes_;
 }
 
 bool ByteBuffer::ReadUInt8(uint8& val) {
@@ -163,4 +163,4 @@ void ByteBuffer::Shift(size_t size) {
   start_ = 0;
 }
 
-} // namespace cricket
+} // namespace talk_base
