@@ -25,19 +25,18 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef __MESSAGEQUEUE_H__
-#define __MESSAGEQUEUE_H__
+#ifndef TALK_BASE_MESSAGEQUEUE_H__
+#define TALK_BASE_MESSAGEQUEUE_H__
 
+#include <vector>
+#include <queue>
+#include <algorithm>
 #include "talk/base/basictypes.h"
 #include "talk/base/criticalsection.h"
 #include "talk/base/socketserver.h"
 #include "talk/base/time.h"
-#include <vector>
-#include <queue>
-#include <algorithm>
-#include <cstring>
 
-namespace cricket {
+namespace talk_base {
 
 struct Message;
 class MessageQueue;
@@ -58,6 +57,7 @@ private:
   ~MessageQueueManager();
 
   static MessageQueueManager* instance_;
+  // This list contains 'active' MessageQueues.
   std::vector<MessageQueue *> message_queues_;
   CriticalSection crit_;
 };
@@ -82,19 +82,38 @@ public:
   virtual ~MessageData() {}
 };
 
-template <class arg1_type>
+template <class T>
 class TypedMessageData : public MessageData {
 public:
-  TypedMessageData(arg1_type data) {
-    data_ = data;
-  }
-  arg1_type data() {
-    return data_;
-  }
+  TypedMessageData(const T& data) : data_(data) { }
+  const T& data() const { return data_; }
+  T& data() { return data_; }
 private:
-  arg1_type data_;
+  T data_;
 };
 
+template<class T>
+inline MessageData* WrapMessageData(const T& data) {
+  return new TypedMessageData<T>(data);
+}
+
+template<class T>
+inline const T& UseMessageData(MessageData* data) {
+  return static_cast< TypedMessageData<T>* >(data)->data();
+}
+
+template<class T>
+class DisposeData : public MessageData {
+public:
+  DisposeData(T* data) : data_(data) { }
+  virtual ~DisposeData() { delete data_; }
+private:
+  T* data_;
+};
+
+const uint32 MQID_ANY = static_cast<uint32>(-1);
+const uint32 MQID_DISPOSE = static_cast<uint32>(-2);
+
 // No destructor
 
 struct Message {
@@ -104,6 +123,7 @@ struct Message {
   MessageHandler *phandler;
   uint32 message_id;
   MessageData *pdata;
+  uint32 ts_sensitive;
 };
 
 // DelayedMessage goes into a priority queue, sorted by trigger time
@@ -133,33 +153,55 @@ public:
   SocketServer* socketserver() { return ss_; }
   void set_socketserver(SocketServer* ss);
 
-  // Once the queue is stopped, all calls to Get/Peek will return false.
+  // Note: The behavior of MessageQueue has changed.  When a MQ is stopped,
+  // futher Posts and Sends will fail.  However, any pending Sends and *ready*
+  // Posts (as opposed to unexpired delayed Posts) will be delivered before
+  // Get (or Peek) returns false.  By guaranteeing delivery of those messages,
+  // we eliminate the race condition when an MessageHandler and MessageQueue
+  // may be destroyed independently of each other.
+
   virtual void Stop();
   virtual bool IsStopping();
   virtual void Restart();
 
-  virtual bool Get(Message *pmsg, int cmsWait = -1);
+  // Get() will process I/O until:
+  //  1) A message is available (returns true)
+  //  2) cmsWait seconds have elapsed (returns false)
+  //  3) Stop() is called (returns false)
+  virtual bool Get(Message *pmsg, int cmsWait = kForever);
   virtual bool Peek(Message *pmsg, int cmsWait = 0);
   virtual void Post(MessageHandler *phandler, uint32 id = 0,
-      MessageData *pdata = NULL);
+      MessageData *pdata = NULL, bool time_sensitive = false);
   virtual void PostDelayed(int cmsDelay, MessageHandler *phandler,
       uint32 id = 0, MessageData *pdata = NULL);
-  virtual void Clear(MessageHandler *phandler, uint32 id = (uint32)-1);
+  virtual void Clear(MessageHandler *phandler, uint32 id = MQID_ANY);
   virtual void Dispatch(Message *pmsg);
   virtual void ReceiveSends();
   virtual int GetDelay();
 
+  // Internally posts a message which causes the doomed object to be deleted
+  template<class T> void Dispose(T* doomed) {
+    if (doomed) {
+      Post(NULL, MQID_DISPOSE, new talk_base::DisposeData<T>(doomed));
+    }
+  }
+
 protected:
+  void EnsureActive();
+
   SocketServer* ss_;
   bool new_ss;
   bool fStop_;
   bool fPeekKeep_;
   Message msgPeek_;
+  // A message queue is active if it has ever had a message posted to it.
+  // This also corresponds to being in MessageQueueManager's global list.
+  bool active_;
   std::queue<Message> msgq_;
   std::priority_queue<DelayedMessage> dmsgq_;
   CriticalSection crit_;
 };
 
-} // namespace cricket 
+} // namespace talk_base
 
-#endif // __MESSAGEQUEUE_H__
+#endif // TALK_BASE_MESSAGEQUEUE_H__
