@@ -140,10 +140,10 @@ static BIO_METHOD methods_socket = {
 	NULL,
 };
 
-BIO_METHOD* BIO_s_socket() { return(&methods_socket); }
+BIO_METHOD* BIO_s_socket2() { return(&methods_socket); }
 
-BIO* BIO_new_socket(cricket::AsyncSocket* socket) {
-	BIO* ret = BIO_new(BIO_s_socket());
+BIO* BIO_new_socket(talk_base::AsyncSocket* socket) {
+	BIO* ret = BIO_new(BIO_s_socket2());
 	if (ret == NULL) {
           return NULL;
 	}
@@ -168,7 +168,7 @@ static int socket_free(BIO* b) {
 static int socket_read(BIO* b, char* out, int outl) {
 	if (!out)
 		return -1;
-	cricket::AsyncSocket* socket = static_cast<cricket::AsyncSocket*>(b->ptr);
+	talk_base::AsyncSocket* socket = static_cast<talk_base::AsyncSocket*>(b->ptr);
 	BIO_clear_retry_flags(b);
   int result = socket->Recv(out, outl);
   if (result > 0) {
@@ -184,7 +184,7 @@ static int socket_read(BIO* b, char* out, int outl) {
 static int socket_write(BIO* b, const char* in, int inl) {
 	if (!in)
 		return -1;
-	cricket::AsyncSocket* socket = static_cast<cricket::AsyncSocket*>(b->ptr);
+	talk_base::AsyncSocket* socket = static_cast<talk_base::AsyncSocket*>(b->ptr);
 	BIO_clear_retry_flags(b);
   int result = socket->Send(in, inl);
   if (result > 0) {
@@ -222,7 +222,7 @@ static long socket_ctrl(BIO* b, int cmd, long num, void* ptr) {
 // OpenSSLAdapter
 /////////////////////////////////////////////////////////////////////////////
 
-namespace cricket {
+namespace talk_base {
 
 OpenSSLAdapter::OpenSSLAdapter(AsyncSocket* socket)
   : SSLAdapter(socket),
@@ -249,7 +249,7 @@ OpenSSLAdapter::StartSSL(const char* hostname, bool restartable) {
     state_ = SSL_WAIT;
     return 0;
   }
-  
+
   state_ = SSL_CONNECTING;
   if (int err = BeginSSL()) {
     Error("BeginSSL", err, false);
@@ -276,12 +276,12 @@ OpenSSLAdapter::BeginSSL() {
     goto ssl_error;
   }
 
-  bio = BIO_new_socket(static_cast<cricket::AsyncSocketAdapter*>(socket_));
+  bio = BIO_new_socket(static_cast<talk_base::AsyncSocketAdapter*>(socket_));
   if (!bio) {
     err = -1;
     goto ssl_error;
   }
- 
+
   ssl_ = SSL_new(ssl_ctx_);
   if (!ssl_) {
     err = -1;
@@ -646,11 +646,19 @@ OpenSSLAdapter::SSLPostConnectionCheck(SSL* ssl, const char* host) {
         break;
 
       void* ext_str = NULL;
+
+#if OPENSSL_VERSION_NUMBER >= 0x0090800fL
+      const unsigned char **ext_value_data = (const_cast<const unsigned char **>
+					      (&extension->value->data));
+#else
+      unsigned char **ext_value_data = &extension->value->data;
+#endif
+
       if (meth->it) {
-        ext_str = ASN1_item_d2i(NULL, (const unsigned char **) &(extension->value->data), extension->value->length,
+        ext_str = ASN1_item_d2i(NULL, ext_value_data, extension->value->length,
                                 ASN1_ITEM_ptr(meth->it));
       } else {
-        ext_str = meth->d2i(NULL, (const unsigned char **) &(extension->value->data), extension->value->length);
+        ext_str = meth->d2i(NULL, ext_value_data, extension->value->length);
       }
 
       STACK_OF(CONF_VALUE)* value = meth->i2v(meth, ext_str, NULL);
@@ -767,12 +775,16 @@ OpenSSLAdapter::SetupSSLContext() {
   SSL_CTX* ctx = SSL_CTX_new(TLSv1_client_method());
   if (ctx == NULL) 
 	  return NULL;
- 
+
   // Add the root cert to the SSL context
-  unsigned char* cert_buffer
+#if OPENSSL_VERSION_NUMBER >= 0x0090800fL
+   const unsigned char* cert_buffer
+#else
+   unsigned char* cert_buffer
+#endif
     = EquifaxSecureGlobalEBusinessCA1_certificate;
   size_t cert_buffer_len = sizeof(EquifaxSecureGlobalEBusinessCA1_certificate);
-  X509* cert = d2i_X509(NULL, (const unsigned char **) &cert_buffer, cert_buffer_len);
+  X509* cert = d2i_X509(NULL, &cert_buffer, cert_buffer_len);
   if (cert == NULL) {
     SSL_CTX_free(ctx);
     return NULL;
@@ -794,4 +806,4 @@ OpenSSLAdapter::SetupSSLContext() {
   return ctx;
 }
 
-} // namespace cricket
+} // namespace talk_base
