@@ -25,8 +25,8 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef __STRINGUTILS_H__
-#define __STRINGUTILS_H__
+#ifndef TALK_BASE_STRINGUTILS_H__
+#define TALK_BASE_STRINGUTILS_H__
 
 #include <ctype.h>
 #include <stdarg.h>
@@ -36,7 +36,17 @@
 #endif  // WIN32
 
 #include <string>
-#include <cstring>
+
+///////////////////////////////////////////////////////////////////////////////
+// Generic string/memory utilities
+///////////////////////////////////////////////////////////////////////////////
+
+namespace talk_base {
+
+// Complement to memset.  Verifies memory consists of count bytes of value c.
+bool memory_check(const void* memory, int c, size_t count);
+
+}  // namespace talk_base
 
 ///////////////////////////////////////////////////////////////////////////////
 // Rename a bunch of common string functions so they are consistent across
@@ -72,6 +82,9 @@ inline int strnicmp(const wchar_t* s1, const wchar_t* s2, size_t n) {
 inline const wchar_t* strchr(const wchar_t* s, wchar_t c) {
   return wcschr(s, c);
 }
+inline const wchar_t* strstr(const wchar_t* haystack, const wchar_t* needle) {
+  return wcsstr(haystack, needle);
+}
 inline int vsnprintf(char* buf, size_t n, const char* fmt, va_list args) {
   return _vsnprintf(buf, n, fmt, args);
 }
@@ -102,7 +115,7 @@ inline int _strnicmp(const char* s1, const char* s2, size_t n) {
 // Traits simplifies porting string functions to be CTYPE-agnostic
 ///////////////////////////////////////////////////////////////////////////////
 
-namespace cricket {
+namespace talk_base {
 
 const size_t SIZE_UNKNOWN = static_cast<size_t>(-1);
 
@@ -256,7 +269,7 @@ size_t asccpyn(wchar_t* buffer, size_t buflen,
 template<>
 struct Traits<char> {
   typedef std::string string;
-  inline static const char* empty_str() { return ""; }
+  inline static const char* Traits<char>::empty_str() { return ""; }
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -273,6 +286,6 @@ struct Traits<wchar_t> {
 
 #endif  // WIN32
 
-}  // namespace cricket
+}  // namespace talk_base
 
-#endif // __STRINGUTILS_H__
+#endif // TALK_BASE_STRINGUTILS_H__
