@@ -28,19 +28,22 @@
 #if defined(_MSC_VER) && _MSC_VER < 1300
 #pragma warning(disable:4786)
 #endif
-#include "talk/base/logging.h"
-#include "talk/base/asyncudpsocket.h"
-#include "talk/base/asynctcpsocket.h"
-#include "talk/base/socketadapters.h"
-#include "talk/p2p/base/port.h"
-#include "talk/p2p/base/helpers.h"
-#include "talk/base/scoped_ptr.h"
+
 #include <errno.h>
+
 #include <algorithm>
 #include <iostream>
-#include <cassert>
 #include <vector>
 
+#include "talk/base/asyncudpsocket.h"
+#include "talk/base/asynctcpsocket.h"
+#include "talk/base/helpers.h"
+#include "talk/base/logging.h"
+#include "talk/base/scoped_ptr.h"
+#include "talk/base/socketadapters.h"
+#include "talk/p2p/base/common.h"
+#include "talk/p2p/base/port.h"
+
 #if defined(_MSC_VER) && _MSC_VER < 1300
 namespace std {
   using ::memcmp;
@@ -108,7 +111,7 @@ inline uint32 ConservativeRTTEstimate(uint32 rtt, uint32 rtt_data_points) {
   if (rtt_data_points == 0)
     return DEFAULT_RTT;
   else
-    return cricket::_max(MINIMUM_RTT, cricket::_min(MAXIMUM_RTT, 2 * rtt));
+    return talk_base::_max(MINIMUM_RTT, talk_base::_min(MAXIMUM_RTT, 2 * rtt));
 }
 
 // Weighting of the old rtt value to new data.
@@ -124,13 +127,29 @@ const uint32 MSG_DELETE = 1;
 
 namespace cricket {
 
-std::string Port::agent_;
-ProxyInfo Port::proxy_;
+static const char * const PROTO_NAMES[PROTO_LAST+1] = { "udp", "tcp", "ssltcp" };
 
-Port::Port(Thread* thread, const std::string& type, SocketFactory* factory,
-           Network* network)
+const char * ProtoToString(ProtocolType proto) {
+  return PROTO_NAMES[proto];
+}
+
+bool StringToProto(const char * value, ProtocolType& proto) {
+  for (size_t i=0; i<=PROTO_LAST; ++i) {
+    if (strcmp(PROTO_NAMES[i], value) == 0) {
+      proto = static_cast<ProtocolType>(i);
+      return true;
+    }
+  }
+  return false;
+}
+
+std::string Port::agent_;
+talk_base::ProxyInfo Port::proxy_;
+
+Port::Port(talk_base::Thread* thread, const std::string& type, 
+           talk_base::SocketFactory* factory, talk_base::Network* network)
   : thread_(thread), factory_(factory), type_(type), network_(network),
-    preference_(-1), lifetime_(LT_PRESTART) {
+    preference_(-1), lifetime_(LT_PRESTART), enable_port_packets_(false) {
 
   if (factory_ == NULL)
     factory_ = thread_->socketserver();
@@ -155,7 +174,7 @@ Port::~Port() {
     delete list[i];
 }
 
-Connection* Port::GetConnection(const SocketAddress& remote_addr) {
+Connection* Port::GetConnection(const talk_base::SocketAddress& remote_addr) {
   AddressMap::const_iterator iter = connections_.find(remote_addr);
   if (iter != connections_.end())
     return iter->second;
@@ -163,15 +182,9 @@ Connection* Port::GetConnection(const SocketAddress& remote_addr) {
     return NULL;
 }
 
-void Port::set_username_fragment(const std::string& username_fragment) {
-  username_frag_ = username_fragment;
-}
-
-void Port::set_password(const std::string& password) {
-  password_ = password;
-}
-
-void Port::add_address(const SocketAddress& address, const std::string& protocol, bool final) {
+void Port::AddAddress(const talk_base::SocketAddress& address,
+                      const std::string& protocol,
+                      bool final) {
   Candidate c;
   c.set_name(name_);
   c.set_type(type_);
@@ -195,22 +208,28 @@ void Port::AddConnection(Connection* conn) {
 }
 
 void Port::OnReadPacket(
-    const char* data, size_t size, const SocketAddress& addr) {
+    const char* data, size_t size, const talk_base::SocketAddress& addr) {
+  // If the user has enabled port packets, just hand this over.
+  if (enable_port_packets_) {
+    SignalReadPacket(this, data, size, addr);
+    return;
+  }
 
   // If this is an authenticated STUN request, then signal unknown address and
   // send back a proper binding response.
   StunMessage* msg;
   std::string remote_username;
   if (!GetStunMessage(data, size, addr, msg, remote_username)) {
-    LOG(LERROR) << "Received non-STUN packet from unknown address: "
-               << addr.ToString();
+    LOG_J(LS_ERROR, this) << "Received non-STUN packet from unknown address ("
+                          << addr.ToString() << ")";
   } else if (!msg) {
     // STUN message handled already
   } else if (msg->type() == STUN_BINDING_REQUEST) {
     SignalUnknownAddress(this, addr, msg, remote_username);
   } else {
-    LOG(LERROR) << "Received unexpected STUN message type (" << msg->type()
-               << ") from unknown address: " << addr.ToString();
+    LOG_J(LS_ERROR, this) << "Received unexpected STUN message type ("
+                          << msg->type() << ") from unknown address ("
+                          << addr.ToString() << ")";
     delete msg;
   }
 }
@@ -232,14 +251,14 @@ void Port::SendBindingRequest(Connection* conn) {
 
   // Send the request message.
   // NOTE: If we wanted to, this is where we would add the HMAC.
-  ByteBuffer buf;
+  talk_base::ByteBuffer buf;
   request.Write(&buf);
   SendTo(buf.Data(), buf.Length(), conn->remote_candidate().address(), false);
 }
 
 bool Port::GetStunMessage(const char* data, size_t size,
-                          const SocketAddress& addr, StunMessage *& msg,
-                          std::string& remote_username) {
+                          const talk_base::SocketAddress& addr, 
+                          StunMessage *& msg, std::string& remote_username) {
   // NOTE: This could clearly be optimized to avoid allocating any memory.
   //       However, at the data rates we'll be looking at on the client side,
   //       this probably isn't worth worrying about.
@@ -248,8 +267,8 @@ bool Port::GetStunMessage(const char* data, size_t size,
 
   // Parse the request message.  If the packet is not a complete and correct
   // STUN message, then ignore it.
-  buzz::scoped_ptr<StunMessage> stun_msg(new StunMessage());
-  ByteBuffer buf(data, size);
+  talk_base::scoped_ptr<StunMessage> stun_msg(new StunMessage());
+  talk_base::ByteBuffer buf(data, size);
   if (!stun_msg->Read(&buf) || (buf.Length() > 0)) {
     return false;
   }
@@ -267,7 +286,7 @@ bool Port::GetStunMessage(const char* data, size_t size,
     if ((remote_frag_len < 0)
         || (std::memcmp(username_attr->bytes(),
                         username_frag_.c_str(), username_frag_.size()) != 0)) {
-      LOG(LERROR) << "Received STUN request with bad username";
+      LOG_J(LS_ERROR, this) << "Received STUN request with bad username";
       SendBindingErrorResponse(stun_msg.get(), addr, STUN_ERROR_BAD_REQUEST,
         STUN_ERROR_REASON_BAD_REQUEST);
       return true;
@@ -280,7 +299,7 @@ bool Port::GetStunMessage(const char* data, size_t size,
     if ((remote_frag_len < 0)
         || (std::memcmp(username_attr->bytes() + remote_frag_len,
                         username_frag_.c_str(), username_frag_.size()) != 0)) {
-      LOG(LERROR) << "Received STUN response with bad username";
+       LOG_J(LS_ERROR, this) << "Received STUN response with bad username";
       // Do not send error response to a response
       return true;
     }
@@ -290,20 +309,21 @@ bool Port::GetStunMessage(const char* data, size_t size,
 
     if (stun_msg->type() == STUN_BINDING_ERROR_RESPONSE) {
       if (const StunErrorCodeAttribute* error_code = stun_msg->GetErrorCode()) {
-        LOG(LERROR) << "Received STUN binding error:"
-                   << " class=" << error_code->error_class()
-                   << " number=" << error_code->number()
-                   << " reason='" << error_code->reason() << "'";
+        LOG_J(LS_ERROR, this) << "Received STUN binding error:"
+                              << " class=" << error_code->error_class()
+                              << " number=" << error_code->number()
+                              << " reason='" << error_code->reason() << "'";
         // Return message to allow error-specific processing
       } else {
-        LOG(LERROR) << "Received STUN error response with no error code";
+        LOG_J(LS_ERROR, this)
+          << "Received STUN error response with no error code";
         // Drop corrupt message
         return true;
       }
     }
   } else {
-    LOG(LERROR) << "Received STUN packet with invalid type: "
-               << stun_msg->type();
+    LOG_J(LS_ERROR, this) << "Received STUN packet with invalid type ("
+                          << stun_msg->type() << ")";
     return true;
   }
 
@@ -313,7 +333,7 @@ bool Port::GetStunMessage(const char* data, size_t size,
 }
 
 void Port::SendBindingResponse(
-    StunMessage* request, const SocketAddress& addr) {
+    StunMessage* request, const talk_base::SocketAddress& addr) {
 
   assert(request->type() == STUN_BINDING_REQUEST);
 
@@ -342,7 +362,7 @@ void Port::SendBindingResponse(
 
   // Send the response message.
   // NOTE: If we wanted to, this is where we would add the HMAC.
-  ByteBuffer buf;
+  talk_base::ByteBuffer buf;
   response.Write(&buf);
   SendTo(buf.Data(), buf.Length(), addr, false);
 
@@ -355,7 +375,7 @@ void Port::SendBindingResponse(
 }
 
 void Port::SendBindingErrorResponse(
-    StunMessage* request, const SocketAddress& addr, int error_code,
+    StunMessage* request, const talk_base::SocketAddress& addr, int error_code,
     const std::string& reason) {
 
   assert(request->type() == STUN_BINDING_REQUEST);
@@ -384,46 +404,58 @@ void Port::SendBindingErrorResponse(
 
   // Send the response message.
   // NOTE: If we wanted to, this is where we would add the HMAC.
-  ByteBuffer buf;
+  talk_base::ByteBuffer buf;
   response.Write(&buf);
   SendTo(buf.Data(), buf.Length(), addr, false);
 }
 
-AsyncPacketSocket * Port::CreatePacketSocket(ProtocolType proto) {
+talk_base::AsyncPacketSocket* Port::CreatePacketSocket(ProtocolType proto) {
   if (proto == PROTO_UDP) {
-    return new AsyncUDPSocket(factory_->CreateAsyncSocket(SOCK_DGRAM));
+    return new talk_base::AsyncUDPSocket(
+        factory_->CreateAsyncSocket(SOCK_DGRAM));
   } else if ((proto == PROTO_TCP) || (proto == PROTO_SSLTCP)) {
-    AsyncSocket * socket = factory_->CreateAsyncSocket(SOCK_STREAM);
+    talk_base::AsyncSocket * socket = factory_->CreateAsyncSocket(SOCK_STREAM);
     switch (proxy().type) {
-    case PROXY_NONE:
+    case talk_base::PROXY_NONE:
       break;
-    case PROXY_SOCKS5:
-      socket = new AsyncSocksProxySocket(socket, proxy().address,
-                                         proxy().username, proxy().password);
+    case talk_base::PROXY_SOCKS5:
+      socket = new talk_base::AsyncSocksProxySocket(
+          socket, proxy().address, proxy().username, proxy().password);
       break;
-    case PROXY_HTTPS:
+    case talk_base::PROXY_HTTPS:
     default:
-      socket = new AsyncHttpsProxySocket(socket, user_agent(), proxy().address,
-                                         proxy().username, proxy().password);
+      socket = new talk_base::AsyncHttpsProxySocket(
+          socket, user_agent(), proxy().address, proxy().username, 
+          proxy().password);
       break;
     }
     if (proto == PROTO_SSLTCP) {
-      socket = new AsyncSSLSocket(socket);
+      socket = new talk_base::AsyncSSLSocket(socket);
     }
-    return new AsyncTCPSocket(socket);
+    return new talk_base::AsyncTCPSocket(socket);
   } else {
-    LOG(INFO) << "Unknown protocol: " << proto;
+    LOG_J(LS_INFO, this) << "Unknown protocol (" << proto << ")";
     return 0;
   }
 }
 
-void Port::OnMessage(Message *pmsg) {
+void Port::OnMessage(talk_base::Message *pmsg) {
   assert(pmsg->message_id == MSG_CHECKTIMEOUT);
   assert(lifetime_ == LT_PRETIMEOUT);
   lifetime_ = LT_POSTTIMEOUT;
   CheckTimeout();
 }
 
+std::string Port::ToString() const {
+  std::stringstream ss;
+  ss << "Port[" << name_ << ":" << type_ << ":" << network_->ToString() << "]";
+  return ss.str();
+}
+
+void Port::EnablePortPackets() {
+  enable_port_packets_ = true;
+}
+
 void Port::Start() {
   // The port sticks around for a minimum lifetime, after which
   // we destroy it when it drops to zero connections.
@@ -431,7 +463,7 @@ void Port::Start() {
     lifetime_ = LT_PRETIMEOUT;
     thread_->PostDelayed(kPortTimeoutDelay, this, MSG_CHECKTIMEOUT);
   } else {
-    LOG(WARNING) << "Port restart attempted";
+    LOG_J(LS_WARNING, this) << "Port restart attempted";
   }
 }
 
@@ -443,15 +475,20 @@ void Port::OnConnectionDestroyed(Connection* conn) {
   CheckTimeout();
 }
 
+void Port::Destroy() {
+  assert(connections_.empty());
+  LOG_J(LS_INFO, this) << "Port deleted";
+  SignalDestroyed(this);
+  delete this;
+}
+
 void Port::CheckTimeout() {
   // If this port has no connections, then there's no reason to keep it around.
   // When the connections time out (both read and write), they will delete
   // themselves, so if we have any connections, they are either readable or
   // writable (or still connecting).
   if ((lifetime_ == LT_POSTTIMEOUT) && connections_.empty()) {
-    LOG(INFO) << "Destroying port: " << name_ << "-" << type_;
-    SignalDestroyed(this);
-    delete this;
+    Destroy();
   }
 }
 
@@ -501,14 +538,15 @@ private:
 //
 
 Connection::Connection(Port* port, size_t index, const Candidate& remote_candidate)
-  : port_(port), local_candidate_index_(index), remote_candidate_(remote_candidate),
-    read_state_(STATE_READ_TIMEOUT), write_state_(STATE_WRITE_CONNECT),
-    connected_(true), pruned_(false),requests_(port->thread()), rtt_(0),
-    rtt_data_points_(0), last_ping_sent_(0), last_ping_received_(0),
+  : requests_(port->thread()), port_(port), local_candidate_index_(index),
+    remote_candidate_(remote_candidate), read_state_(STATE_READ_TIMEOUT),
+    write_state_(STATE_WRITE_CONNECT), connected_(true), pruned_(false),
+    rtt_(0), rtt_data_points_(0), last_ping_sent_(0), last_ping_received_(0),
     recv_total_bytes_(0), recv_bytes_second_(0),
     last_recv_bytes_second_time_((uint32)-1), last_recv_bytes_second_calc_(0),
     sent_total_bytes_(0), sent_bytes_second_(0),
-    last_sent_bytes_second_time_((uint32)-1), last_sent_bytes_second_calc_(0) {
+    last_sent_bytes_second_time_((uint32)-1), last_sent_bytes_second_calc_(0),
+    reported_(false) {
 
   // Wire up to send stun packets
   requests_.SignalSendPacket.connect(this, &Connection::OnSendStunPacket);
@@ -529,6 +567,7 @@ void Connection::set_read_state(ReadState value) {
   ReadState old_value = read_state_;
   read_state_ = value;
   if (value != old_value) {
+    LOG_J(LS_VERBOSE, this) << "set_read_state";
     SignalStateChange(this);
     CheckTimeout();
   }
@@ -538,6 +577,7 @@ void Connection::set_write_state(WriteState value) {
   WriteState old_value = write_state_;
   write_state_ = value;
   if (value != old_value) {
+    LOG_J(LS_VERBOSE, this) << "set_write_state";
     SignalStateChange(this);
     CheckTimeout();
   }
@@ -546,20 +586,20 @@ void Connection::set_write_state(WriteState value) {
 void Connection::set_connected(bool value) {
   bool old_value = connected_;
   connected_ = value;
-
-  // When connectedness is turned off, this connection is done.
-  if (old_value && !value)
-    set_write_state(STATE_WRITE_TIMEOUT);
+  if (value != old_value) {
+    LOG_J(LS_VERBOSE, this) << "set_connected";
+  }
 }
 
-void Connection::OnSendStunPacket(const void* data, size_t size) {
+void Connection::OnSendStunPacket(
+    const void* data, size_t size, StunRequest* req) {
   port_->SendTo(data, size, remote_candidate_.address(), false);
 }
 
 void Connection::OnReadPacket(const char* data, size_t size) {
   StunMessage* msg;
   std::string remote_username;
-  const SocketAddress& addr(remote_candidate_.address());
+  const talk_base::SocketAddress& addr(remote_candidate_.address());
   if (!port_->GetStunMessage(data, size, addr, msg, remote_username)) {
     // The packet did not parse as a valid STUN message
   
@@ -578,13 +618,14 @@ void Connection::OnReadPacket(const char* data, size_t size) {
       // Not readable means the remote address hasn't send a valid
       // binding request yet.
 
-      LOG(WARNING) << "Received non-STUN packet from an unreadable connection.";
+      LOG_J(LS_WARNING, this)
+        << "Received non-STUN packet from an unreadable connection.";
     }
   } else if (!msg) {
     // The packet was STUN, but was already handled
   } else if (remote_username != remote_candidate_.username()) {
     // Not destined this connection
-    LOG(LERROR) << "Received STUN packet on wrong address.";
+    LOG_J(LS_ERROR, this) << "Received STUN packet on wrong address.";
     if (msg->type() == STUN_BINDING_REQUEST) {
       port_->SendBindingErrorResponse(msg, addr, STUN_ERROR_BAD_REQUEST,
                                       STUN_ERROR_REASON_BAD_REQUEST);
@@ -627,12 +668,16 @@ void Connection::OnReadPacket(const char* data, size_t size) {
 }
 
 void Connection::Prune() {
-  pruned_ = true;
-  requests_.Clear();
-  set_write_state(STATE_WRITE_TIMEOUT);
+  if (!pruned_) {
+    LOG_J(LS_VERBOSE, this) << "Connection pruned";
+    pruned_ = true;
+    requests_.Clear();
+    set_write_state(STATE_WRITE_TIMEOUT);
+  }
 }
 
 void Connection::Destroy() {
+  LOG_J(LS_VERBOSE, this) << "Connection destroyed";
   set_read_state(STATE_READ_TIMEOUT);
   set_write_state(STATE_WRITE_TIMEOUT);
 }
@@ -686,10 +731,38 @@ void Connection::Ping(uint32 now) {
 }
 
 void Connection::ReceivedPing() {
-  last_ping_received_ = Time();
+  last_ping_received_ = talk_base::Time();
   set_read_state(STATE_READABLE);
 }
 
+std::string Connection::ToString() const {
+  const char CONNECT_STATE_ABBREV[2] = {
+    '-', // not connected (false)
+    'C', // connected (true)
+  };
+  const char READ_STATE_ABBREV[2] = {
+    'R', // STATE_READABLE
+    '-', // STATE_READ_TIMEOUT
+  };
+  const char WRITE_STATE_ABBREV[3] = {
+    'W', // STATE_WRITABLE
+    'w', // STATE_WRITE_CONNECT
+    '-', // STATE_WRITE_TIMEOUT
+  };
+  const Candidate& local = local_candidate();
+  const Candidate& remote = remote_candidate();
+  std::stringstream ss;
+  ss << "Conn[" << local.generation()
+     << ":" << local.name() << ":" << local.type() << ":" << local.address().ToString()
+     << "->" << remote.name() << ":" << remote.type() << ":" << remote.address().ToString()
+     << "|" 
+     << CONNECT_STATE_ABBREV[connected()]
+     << READ_STATE_ABBREV[read_state()]
+     << WRITE_STATE_ABBREV[write_state()]
+     << "]";
+  return ss.str();
+}
+
 void Connection::OnConnectionRequestResponse(StunMessage *response, uint32 rtt) {
   // We have a potentially valid reply from the remote address.
   // The packet must include a username that ends with our fragment,
@@ -705,7 +778,7 @@ void Connection::OnConnectionRequestResponse(StunMessage *response, uint32 rtt)
       response->GetByteString(STUN_ATTR_USERNAME);
   if (valid) {
     if (!username_attr) {
-      LOG(LERROR) << "Received likely STUN packet with no username";
+      LOG_J(LS_ERROR, this) << "Received likely STUN packet with no username";
       valid = false;
     }
   }
@@ -714,7 +787,7 @@ void Connection::OnConnectionRequestResponse(StunMessage *response, uint32 rtt)
   // be bigger since our fragment is at the end!)
   if (valid) {
     if (username_attr->length() <= port_->username_fragment().size()) {
-      LOG(LERROR) << "Received likely STUN packet with short username";
+      LOG_J(LS_ERROR, this) << "Received likely STUN packet with short username";
       valid = false;
     }
   }
@@ -725,7 +798,7 @@ void Connection::OnConnectionRequestResponse(StunMessage *response, uint32 rtt)
     int offset = (int)(username_attr->length() - username_fragment.size());
     if (std::memcmp(username_attr->bytes() + offset,
         username_fragment.c_str(), username_fragment.size()) != 0) {
-      LOG(LERROR) << "Received STUN response with bad username";
+      LOG_J(LS_ERROR, this) << "Received STUN response with bad username";
       valid = false;
     }
   }
@@ -754,7 +827,7 @@ void Connection::OnConnectionRequestErrorResponse(StunMessage *response, uint32
     // Race failure, retry
   } else {
     // This is not a valid connection.
-    set_connected(false);
+    set_write_state(STATE_WRITE_TIMEOUT);
   }
 }
 
@@ -769,13 +842,10 @@ void Connection::CheckTimeout() {
   }
 }
 
-void Connection::OnMessage(Message *pmsg) {
+void Connection::OnMessage(talk_base::Message *pmsg) {
   assert(pmsg->message_id == MSG_DELETE);
 
-  LOG(INFO) << "Destroying connection: from "
-            << local_candidate().address().ToString()
-            << " to " << remote_candidate_.address().ToString();
-
+  LOG_J(LS_INFO, this) << "Connection deleted";
   SignalDestroyed(this);
   delete this;
 }
@@ -783,9 +853,9 @@ void Connection::OnMessage(Message *pmsg) {
 size_t Connection::recv_bytes_second() {
   // Snapshot bytes / second calculator
 
-  uint32 current_time = Time();
+  uint32 current_time = talk_base::Time();
   if (last_recv_bytes_second_time_ != (uint32)-1) {
-    int delta = TimeDiff(current_time, last_recv_bytes_second_time_);
+    int delta = talk_base::TimeDiff(current_time, last_recv_bytes_second_time_);
     if (delta >= 1000) {
       int fraction_time = delta % 1000;
       int seconds_time = delta - fraction_time;
@@ -810,9 +880,9 @@ size_t Connection::recv_total_bytes() {
 size_t Connection::sent_bytes_second() {
   // Snapshot bytes / second calculator
 
-  uint32 current_time = Time();
+  uint32 current_time = talk_base::Time();
   if (last_sent_bytes_second_time_ != (uint32)-1) {
-    int delta = TimeDiff(current_time, last_sent_bytes_second_time_);
+    int delta = talk_base::TimeDiff(current_time, last_sent_bytes_second_time_);
     if (delta >= 1000) {
       int fraction_time = delta % 1000;
       int seconds_time = delta - fraction_time;
