@@ -30,8 +30,8 @@
 #endif
 #include "talk/base/logging.h"
 #include "talk/base/asynctcpsocket.h"
+#include "talk/base/helpers.h"
 #include "talk/p2p/base/relayport.h"
-#include "talk/p2p/base/helpers.h"
 #include <iostream>
 #include <cassert>
 #ifdef OSX
@@ -50,50 +50,49 @@ extern "C" {
 }
 #endif // POSIX
 
+namespace talk_base {
+class AsyncTCPSocket;
+};
+
 namespace cricket {
 
 const int KEEPALIVE_DELAY = 10 * 60 * 1000;
 const int RETRY_DELAY = 50; // 50ms, from ICE spec
 const uint32 RETRY_TIMEOUT = 50 * 1000; // ICE says 50 secs
 
-const uint32 MSG_DISPOSE_SOCKET = 100; // needs to be more than ID used by Port
-typedef TypedMessageData<AsyncPacketSocket *> DisposeSocketData;
-
-class AsyncTCPSocket;
-
 // Manages a single connection to the relayserver.  We aim to use each
 // connection for only a specific destination address so that we can avoid
 // wrapping every packet in a STUN send / data indication.
 class RelayEntry : public sigslot::has_slots<> {
 public:
-  RelayEntry(RelayPort* port, const SocketAddress& ext_addr, const SocketAddress& local_addr);
+  RelayEntry(RelayPort* port, const talk_base::SocketAddress& ext_addr, 
+    const talk_base::SocketAddress& local_addr);
   ~RelayEntry();
 
   RelayPort* port() { return port_; }
 
-  const SocketAddress& address() { return ext_addr_; }
-  void set_address(const SocketAddress& addr) { ext_addr_ = addr; }
+  const talk_base::SocketAddress& address() const { return ext_addr_; }
+  void set_address(const talk_base::SocketAddress& addr) { ext_addr_ = addr; }
 
-  AsyncPacketSocket* socket() { return socket_; }
+  talk_base::AsyncPacketSocket* socket() { return socket_; }
 
-  bool connected() { return connected_; }
-  void set_connected(bool connected) { connected_ = connected; }
-
-  bool locked() { return locked_; }
+  bool connected() const { return connected_; }
+  bool locked() const { return locked_; }
 
   // Returns the last error on the socket of this entry.
-  int GetError() { return socket_->GetError(); }
+  int GetError() const { return socket_->GetError(); }
 
   // Sends the STUN requests to the server to initiate this connection.
   void Connect();
 
   // Called when this entry becomes connected.  The address given is the one
   // exposed to the outside world on the relay server.
-  void OnConnect(const SocketAddress& mapped_addr);
+  void OnConnect(const talk_base::SocketAddress& mapped_addr);
 
   // Sends a packet to the given destination address using the socket of this
   // entry.  This will wrap the packet in STUN if necessary.
-  int SendTo(const void* data, size_t size, const SocketAddress& addr);
+  int SendTo(const void* data, size_t size, 
+    const talk_base::SocketAddress& addr);
 
   // Schedules a keep-alive allocate request.
   void ScheduleKeepAlive();
@@ -106,25 +105,26 @@ public:
 
 private:
   RelayPort* port_;
-  SocketAddress ext_addr_, local_addr_;
+  talk_base::SocketAddress ext_addr_, local_addr_;
   size_t server_index_;
-  AsyncPacketSocket* socket_;
+  talk_base::AsyncPacketSocket* socket_;
   bool connected_;
   bool locked_;
   StunRequestManager requests_;
 
   // Called when a TCP connection is established or fails
-  void OnSocketConnect(AsyncTCPSocket* socket);
-  void OnSocketClose(AsyncTCPSocket* socket, int error);
+  void OnSocketConnect(talk_base::AsyncTCPSocket* socket);
+  void OnSocketClose(talk_base::AsyncTCPSocket* socket, int error);
 
   // Called when a packet is received on this socket.
   void OnReadPacket(
-      const char* data, size_t size, const SocketAddress& remote_addr,
-      AsyncPacketSocket* socket);
+      const char* data, size_t size, 
+      const talk_base::SocketAddress& remote_addr,
+      talk_base::AsyncPacketSocket* socket);
 
   // Called on behalf of a StunRequest to write data to the socket.  This is
   // already STUN intended for the server, so no wrapping is necessary.
-  void OnSendPacket(const void* data, size_t size);
+  void OnSendPacket(const void* data, size_t size, StunRequest* req);
 
   // Sends the given data on the socket to the server with no wrapping.  This
   // returns the number of bytes written or -1 if an error occurred.
@@ -153,13 +153,15 @@ private:
 const std::string RELAY_PORT_TYPE("relay");
 
 RelayPort::RelayPort(
-    Thread* thread, SocketFactory* factory, Network* network,
-    const SocketAddress& local_addr, const std::string& username,
-    const std::string& password, const std::string& magic_cookie)
+    talk_base::Thread* thread, talk_base::SocketFactory* factory, 
+    talk_base::Network* network, const talk_base::SocketAddress& local_addr, 
+    const std::string& username, const std::string& password, 
+    const std::string& magic_cookie)
   : Port(thread, RELAY_PORT_TYPE, factory, network), local_addr_(local_addr),
     ready_(false), magic_cookie_(magic_cookie), error_(0) {
 
-  entries_.push_back(new RelayEntry(this, SocketAddress(), local_addr_));
+  entries_.push_back(
+    new RelayEntry(this, talk_base::SocketAddress(), local_addr_));
 
   set_username_fragment(username);
   set_password(password);
@@ -177,7 +179,8 @@ RelayPort::~RelayPort() {
 void RelayPort::AddServerAddress(const ProtocolAddress& addr) {
   // Since HTTP proxies usually only allow 443, let's up the priority on PROTO_SSLTCP
   if ((addr.proto == PROTO_SSLTCP)
-      && ((proxy().type == PROXY_HTTPS) || (proxy().type == PROXY_UNKNOWN))) {
+      && ((proxy().type == talk_base::PROXY_HTTPS) 
+          || (proxy().type == talk_base::PROXY_UNKNOWN))) {
     server_addr_.push_front(addr);
   } else {
     server_addr_.push_back(addr);
@@ -192,7 +195,7 @@ void RelayPort::AddExternalAddress(const ProtocolAddress& addr) {
       return;
     }
   }
-  add_address(addr.address, proto_name, false);
+  AddAddress(addr.address, proto_name, false);
 }
 
 void RelayPort::SetReady() {
@@ -203,7 +206,7 @@ void RelayPort::SetReady() {
 }
 
 const ProtocolAddress * RelayPort::ServerAddress(size_t index) const {
-  if (index < server_addr_.size())
+  if ((index >= 0) && (index < server_addr_.size()))
     return &server_addr_[index];
   return 0;
 }
@@ -249,9 +252,8 @@ Connection* RelayPort::CreateConnection(const Candidate& address, CandidateOrigi
   return conn;
 }
 
-int RelayPort::SendTo(const void* data,
-                      size_t size,
-                      const SocketAddress& addr, bool payload) {
+int RelayPort::SendTo(const void* data, size_t size,
+                      const talk_base::SocketAddress& addr, bool payload) {
 
   // Try to find an entry for this specific address.  Note that the first entry
   // created was not given an address initially, so it can be set to the first
@@ -307,19 +309,7 @@ int RelayPort::SendTo(const void* data,
   return (int)size;
 }
 
-void RelayPort::OnMessage(Message *pmsg) {
-  switch (pmsg->message_id) {
-  case MSG_DISPOSE_SOCKET: {
-    DisposeSocketData * data = static_cast<DisposeSocketData *>(pmsg->pdata);
-    delete data->data();
-    delete data;
-    break; }
-  default:
-    Port::OnMessage(pmsg);
-  }
-}
-
-int RelayPort::SetOption(Socket::Option opt, int value) {
+int RelayPort::SetOption(talk_base::Socket::Option opt, int value) {
   int result = 0;
   for (unsigned i = 0; i < entries_.size(); ++i) {
     if (entries_[i]->socket()->SetOption(opt, value) < 0) {
@@ -336,7 +326,8 @@ int RelayPort::GetError() {
 }
 
 void RelayPort::OnReadPacket(
-    const char* data, size_t size, const SocketAddress& remote_addr) {
+    const char* data, size_t size, 
+    const talk_base::SocketAddress& remote_addr) {
   if (Connection* conn = GetConnection(remote_addr)) {
     conn->OnReadPacket(data, size);
   } else {
@@ -344,12 +335,13 @@ void RelayPort::OnReadPacket(
   }
 }
 
-void RelayPort::DisposeSocket(AsyncPacketSocket * socket) {
-  thread_->Post(this, MSG_DISPOSE_SOCKET, new DisposeSocketData(socket));
+void RelayPort::DisposeSocket(talk_base::AsyncPacketSocket * socket) {
+  thread_->Dispose(socket);
 }
 
-RelayEntry::RelayEntry(RelayPort* port, const SocketAddress& ext_addr,
-                       const SocketAddress& local_addr)
+RelayEntry::RelayEntry(RelayPort* port, 
+                       const talk_base::SocketAddress& ext_addr,
+                       const talk_base::SocketAddress& local_addr)
   : port_(port), ext_addr_(ext_addr), local_addr_(local_addr), server_index_(0),
     socket_(0), connected_(false), locked_(false), requests_(port->thread()) {
 
@@ -381,7 +373,8 @@ void RelayEntry::Connect() {
     socket_->SetOption(port_->options()[i].first, port_->options()[i].second);
 
   if ((ra->proto == PROTO_TCP) || (ra->proto == PROTO_SSLTCP)) {
-    AsyncTCPSocket * tcp = static_cast<AsyncTCPSocket *>(socket_);
+    talk_base::AsyncTCPSocket * tcp 
+      = static_cast<talk_base::AsyncTCPSocket *>(socket_);
     tcp->SignalClose.connect(this, &RelayEntry::OnSocketClose);
     tcp->SignalConnect.connect(this, &RelayEntry::OnSocketConnect);
     tcp->Connect(ra->address);
@@ -390,7 +383,7 @@ void RelayEntry::Connect() {
   }
 }
 
-void RelayEntry::OnConnect(const SocketAddress& mapped_addr) {
+void RelayEntry::OnConnect(const talk_base::SocketAddress& mapped_addr) {
   ProtocolType proto = PROTO_UDP;
   LOG(INFO) << "Relay allocate succeeded: " << ProtoToString(proto) << " @ " << mapped_addr.ToString();
   connected_ = true;
@@ -399,9 +392,8 @@ void RelayEntry::OnConnect(const SocketAddress& mapped_addr) {
   port_->SetReady();
 }
 
-int RelayEntry::SendTo(const void* data,
-                        size_t size,
-                        const SocketAddress& addr) {
+int RelayEntry::SendTo(const void* data, size_t size,
+                        const talk_base::SocketAddress& addr) {
 
   // If this connection is locked to the address given, then we can send the
   // packet with no wrapper.
@@ -453,7 +445,7 @@ int RelayEntry::SendTo(const void* data,
 
   // TODO: compute the HMAC.
 
-  ByteBuffer buf;
+  talk_base::ByteBuffer buf;
   request.Write(&buf);
 
   return SendPacket(buf.Data(), buf.Length());
@@ -477,22 +469,21 @@ void RelayEntry::HandleConnectFailure() {
   Connect();
 }
 
-void RelayEntry::OnSocketConnect(AsyncTCPSocket* socket) {
+void RelayEntry::OnSocketConnect(talk_base::AsyncTCPSocket* socket) {
   assert(socket == socket_);
   LOG(INFO) << "relay tcp connected to " << socket->GetRemoteAddress().ToString();
   requests_.Send(new AllocateRequest(this));
 }
 
-void RelayEntry::OnSocketClose(AsyncTCPSocket* socket, int error) {
+void RelayEntry::OnSocketClose(talk_base::AsyncTCPSocket* socket, int error) {
   assert(socket == socket_);
   PLOG(LERROR, error) << "relay tcp connect failed";
   HandleConnectFailure();
 }
 
-void RelayEntry::OnReadPacket(const char* data,
-                              size_t size,
-                              const SocketAddress& remote_addr,
-                              AsyncPacketSocket* socket) {
+void RelayEntry::OnReadPacket(const char* data, size_t size,
+                              const talk_base::SocketAddress& remote_addr,
+                              talk_base::AsyncPacketSocket* socket) {
   assert(socket == socket_);
   //assert(remote_addr == port_->server_addr()); TODO: are we worried about this?
 
@@ -507,7 +498,7 @@ void RelayEntry::OnReadPacket(const char* data,
     return;
   }
 
-  ByteBuffer buf(data, size);
+  talk_base::ByteBuffer buf(data, size);
   StunMessage msg;
   if (!msg.Read(&buf)) {
     LOG(INFO) << "Incoming packet was not STUN";
@@ -543,7 +534,7 @@ void RelayEntry::OnReadPacket(const char* data,
     return;
   }
 
-  SocketAddress remote_addr2(addr_attr->ip(), addr_attr->port());
+  talk_base::SocketAddress remote_addr2(addr_attr->ip(), addr_attr->port());
 
   const StunByteStringAttribute* data_attr = msg.GetByteString(STUN_ATTR_DATA);
   if (!data_attr) {
@@ -555,7 +546,7 @@ void RelayEntry::OnReadPacket(const char* data,
   port_->OnReadPacket(data_attr->bytes(), data_attr->length(), remote_addr2);
 }
 
-void RelayEntry::OnSendPacket(const void* data, size_t size) {
+void RelayEntry::OnSendPacket(const void* data, size_t size, StunRequest* req) {
   SendPacket(data, size);
 }
 
@@ -575,7 +566,7 @@ int RelayEntry::SendPacket(const void* data, size_t size) {
 }
 
 AllocateRequest::AllocateRequest(RelayEntry* entry) : entry_(entry) {
-  start_time_ = GetMillisecondCount();
+  start_time_ = talk_base::GetMillisecondCount();
 }
 
 void AllocateRequest::Prepare(StunMessage* request) {
@@ -597,7 +588,7 @@ void AllocateRequest::Prepare(StunMessage* request) {
 }
 
 int AllocateRequest::GetNextDelay() {
-  int delay = 100 * _max(1 << count_, 2);
+  int delay = 100 * talk_base::_max(1 << count_, 2);
   count_ += 1;
   if (count_ == 5)
     timeout_ = true;
@@ -612,7 +603,7 @@ void AllocateRequest::OnResponse(StunMessage* response) {
   } else if (addr_attr->family() != 1) {
     LOG(INFO) << "Mapped address has bad family";
   } else {
-    SocketAddress addr(addr_attr->ip(), addr_attr->port());
+    talk_base::SocketAddress addr(addr_attr->ip(), addr_attr->port());
     entry_->OnConnect(addr);
   }
 
@@ -631,7 +622,7 @@ void AllocateRequest::OnErrorResponse(StunMessage* response) {
               << " reason='" << attr->reason() << "'";
   }
 
-  if (GetMillisecondCount() - start_time_ <= RETRY_TIMEOUT)
+  if (talk_base::GetMillisecondCount() - start_time_ <= RETRY_TIMEOUT)
     entry_->ScheduleKeepAlive();
 }
 
