@@ -45,7 +45,7 @@ const uint32 MSG_CREATEVOICECHANNEL = 1;
 const uint32 MSG_DESTROYVOICECHANNEL = 2;
 const uint32 MSG_SETAUDIOOPTIONS = 3;
 
-ChannelManager::ChannelManager(Thread *worker_thread) {
+ChannelManager::ChannelManager(talk_base::Thread *worker_thread) {
 #ifdef HAVE_GIPS
   media_engine_ = new GipsMediaEngine();
 #elif HAVE_GIPSLITE
@@ -99,13 +99,13 @@ VoiceChannel *ChannelManager::CreateVoiceChannel(Session *session) {
   CreateParams params;
   params.session = session;
   params.channel = NULL;
-  TypedMessageData<CreateParams *> data(&params);
+  talk_base::TypedMessageData<CreateParams *> data(&params);
   worker_thread_->Send(this, MSG_CREATEVOICECHANNEL, &data);
   return params.channel;
 }
 
 VoiceChannel *ChannelManager::CreateVoiceChannel_w(Session *session) {
-  CritScope cs(&crit_);
+  talk_base::CritScope cs(&crit_);
 
   // This is ok to alloc from a thread other than the worker thread
   assert(initialized_);
@@ -119,12 +119,12 @@ VoiceChannel *ChannelManager::CreateVoiceChannel_w(Session *session) {
 }
 
 void ChannelManager::DestroyVoiceChannel(VoiceChannel *voice_channel) {
-  TypedMessageData<VoiceChannel *> data(voice_channel);
+  talk_base::TypedMessageData<VoiceChannel *> data(voice_channel);
   worker_thread_->Send(this, MSG_DESTROYVOICECHANNEL, &data);
 }
 
 void ChannelManager::DestroyVoiceChannel_w(VoiceChannel *voice_channel) {
-  CritScope cs(&crit_);
+  talk_base::CritScope cs(&crit_);
   // Destroy voice channel.
   assert(initialized_);
   std::vector<VoiceChannel *>::iterator it = std::find(channels_.begin(),
@@ -145,15 +145,16 @@ void ChannelManager::SetAudioOptions(bool auto_gain_control, int wave_in_device,
   options.auto_gain_control = auto_gain_control;
   options.wave_in_device = wave_in_device;
   options.wave_out_device = wave_out_device;
-  TypedMessageData<AudioOptions> data(options);
+  talk_base::TypedMessageData<AudioOptions> data(options);
   worker_thread_->Send(this, MSG_SETAUDIOOPTIONS, &data);
 }
 
 void ChannelManager::SetAudioOptions_w(AudioOptions options) {
-  assert(worker_thread_ == Thread::Current());
+  assert(worker_thread_ == talk_base::Thread::Current());
 
   // Set auto gain control on
-  if (media_engine_->SetAudioOptions(options.auto_gain_control?MediaEngine::AUTO_GAIN_CONTROL:0) != 0) {
+  if (media_engine_->SetAudioOptions(
+      options.auto_gain_control ? MediaEngine::AUTO_GAIN_CONTROL : 0) != 0) {
     // TODO: We need to log these failures.
   }
 
@@ -161,14 +162,15 @@ void ChannelManager::SetAudioOptions_w(AudioOptions options) {
   // This will fail if audio is already playing.  Stop all of the media
   // start it up again after changing the setting.
   {
-    CritScope cs(&crit_);
+    talk_base::CritScope cs(&crit_);
     for (VoiceChannels::iterator it = channels_.begin();
          it < channels_.end();
          ++it) {
       (*it)->PauseMedia_w();
     }
 
-    if (media_engine_->SetSoundDevices(options.wave_in_device, options.wave_out_device) == -1) {
+    if (media_engine_->SetSoundDevices(options.wave_in_device, 
+        options.wave_out_device) == -1) {
       // TODO: We need to log these failures.
     }
 
@@ -180,28 +182,34 @@ void ChannelManager::SetAudioOptions_w(AudioOptions options) {
   }
 }
 
-Thread *ChannelManager::worker_thread() {
+talk_base::Thread *ChannelManager::worker_thread() {
   return worker_thread_;
 }
 
-void ChannelManager::OnMessage(Message *message) {
+void ChannelManager::OnMessage(talk_base::Message *message) {
   switch (message->message_id) {
   case MSG_CREATEVOICECHANNEL:
     {
-      TypedMessageData<CreateParams *> *data = static_cast<TypedMessageData<CreateParams *> *>(message->pdata);
+      talk_base::TypedMessageData<CreateParams *> *data 
+        = static_cast<talk_base::TypedMessageData<CreateParams *> *>(
+          message->pdata);
       data->data()->channel = CreateVoiceChannel_w(data->data()->session);
     }
     break;
 
   case MSG_DESTROYVOICECHANNEL:
     {
-      TypedMessageData<VoiceChannel *> *data = static_cast<TypedMessageData<VoiceChannel *> *>(message->pdata);
+      talk_base::TypedMessageData<VoiceChannel *> *data 
+        = static_cast<talk_base::TypedMessageData<VoiceChannel *> *>(
+          message->pdata);
       DestroyVoiceChannel_w(data->data());
     }
     break;
   case MSG_SETAUDIOOPTIONS:
     {
-      TypedMessageData<AudioOptions> *data = static_cast<TypedMessageData<AudioOptions> *>(message->pdata);
+      talk_base::TypedMessageData<AudioOptions> *data 
+        = static_cast<talk_base::TypedMessageData<AudioOptions> *>(
+          message->pdata);
       SetAudioOptions_w(data->data());
     }
     break;
