@@ -33,11 +33,10 @@
 #include "talk/base/sigslot.h"
 #include "talk/base/messagequeue.h"
 #include "talk/base/thread.h"
-#include "talk/p2p/client/sessionclient.h"
 #include "talk/p2p/base/sessionmanager.h"
 #include "talk/p2p/base/session.h"
+#include "talk/p2p/base/sessionclient.h"
 #include "talk/p2p/base/sessiondescription.h"
-#include "talk/xmpp/xmppclient.h"
 #include <map>
 
 namespace cricket {
@@ -45,12 +44,14 @@ namespace cricket {
 class Call;
 class PhoneSessionDescription;
 
-class PhoneSessionClient : public SessionClient {
+class PhoneSessionClient: public SessionClient, public sigslot::has_slots<> {
 public:
   PhoneSessionClient(const buzz::Jid& jid, SessionManager *manager);
   ~PhoneSessionClient();
 
-  const buzz::Jid &jid() const;
+  const buzz::Jid &jid() const { return jid_; }
+  SessionManager* session_manager() const { return session_manager_; }
+  ChannelManager* channel_manager() const { return channel_manager_; }
 
   Call *CreateCall();
   void DestroyCall(Call *call);
@@ -76,13 +77,7 @@ public:
   PhoneSessionDescription* CreateAcceptSessionDescription(const SessionDescription* offer);
 
   // Returns our preference for the given codec.
-  static int GetMediaCodecPreference(const char* name);
-
-  // Returns the name of the first codec in the description that
-  // is found.  Return value is false if none was found.
-  static bool FindMediaCodec(MediaEngine* gips,
-                            const PhoneSessionDescription* desc,
-                            const char **codec);
+  static int GetMediaCodecPreference(Codec codec);
 
 private:
   void OnSessionCreate(Session *session, bool received_initiate);
@@ -90,12 +85,10 @@ private:
   void OnSessionDestroy(Session *session);
   const SessionDescription *CreateSessionDescription(const buzz::XmlElement *element);
   buzz::XmlElement *TranslateSessionDescription(const SessionDescription *description);
-  const std::string &GetSessionDescriptionName();
-  const buzz::Jid &GetJid() const;
   Session *CreateSession(Call *call);
-  ChannelManager *channel_manager();
 
   buzz::Jid jid_;
+  SessionManager* session_manager_;
   Call *focus_call_;
   ChannelManager *channel_manager_;
   std::map<uint32, Call *> calls_;
@@ -107,14 +100,19 @@ private:
 class PhoneSessionDescription: public SessionDescription {
 public:
   // Returns the list of codecs sorted by our preference.
-  const std::vector<MediaEngine::Codec>& codecs() const { return codecs_; }
+  const std::vector<Codec>& codecs() const { return codecs_; }
   // Adds another codec to the list.
-  void AddCodec(const MediaEngine::Codec& codec) { codecs_.push_back(codec); }
+  void AddCodec(const Codec& codec) { codecs_.push_back(codec); }
   // Sorts the list of codecs by preference.
-  void Sort() { std::stable_sort(codecs_.begin(), codecs_.end()); }
+  void Sort() { std::sort(codecs_.begin(), codecs_.end(), PreferenceSort()); }
 
 private:
-  std::vector<MediaEngine::Codec> codecs_;
+  std::vector<Codec> codecs_;
+  struct PreferenceSort {
+    bool operator()(Codec a, Codec b) {
+      return a.preference > b.preference;
+    }
+  };
 
 #if defined(FEATURE_ENABLE_VOICEMAIL)
   std::string lang_;
