@@ -1,8 +1,6 @@
 /*
  * libjingle
  * Copyright 2004--2005, Google Inc.
- * Copyright 2006, INdT
- *  Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -38,12 +36,11 @@
 
 namespace buzz {
 
-Task *
-XmppClient::GetParent(int code) {
+talk_base::Task* XmppClient::GetParent(int code) {
   if (code == XMPP_CLIENT_TASK_CODE)
     return this;
   else
-    return Task::GetParent(code);
+    return talk_base::Task::GetParent(code);
 }
 
 class XmppClient::Private :
@@ -58,7 +55,9 @@ public:
     engine_(NULL),
     proxy_port_(0),
     pre_engine_error_(XmppEngine::ERROR_NONE),
-    signal_closed_(false) {}
+    pre_engine_subcode_(0),
+    signal_closed_(false),
+    allow_plain_(false) {}
 
   // the owner
   XmppClient * const client_;
@@ -67,14 +66,16 @@ public:
   scoped_ptr<AsyncSocket> socket_;
   scoped_ptr<XmppEngine> engine_;
   scoped_ptr<PreXmppAuth> pre_auth_;
-  XmppPassword pass_;
+  talk_base::CryptString pass_;
   std::string auth_cookie_;
-  cricket::SocketAddress server_;
+  talk_base::SocketAddress server_;
   std::string proxy_host_;
   int proxy_port_;
   XmppEngine::Error pre_engine_error_;
+  int pre_engine_subcode_;
   CaptchaChallenge captcha_challenge_;
   bool signal_closed_;
+  bool allow_plain_;
 
   // implementations of interfaces
   void OnStateChange(int state);
@@ -89,7 +90,7 @@ public:
 };
 
 XmppReturnStatus
-XmppClient::Connect(const XmppClientSettings & settings, AsyncSocket * socket, PreXmppAuth * pre_auth) {
+XmppClient::Connect(const XmppClientSettings & settings, const std::string & lang, AsyncSocket * socket, PreXmppAuth * pre_auth) {
   if (socket == NULL)
     return XMPP_RETURN_BADARGUMENT;
   if (d_->socket_.get() != NULL)
@@ -104,21 +105,37 @@ XmppClient::Connect(const XmppClientSettings & settings, AsyncSocket * socket, P
   d_->engine_.reset(XmppEngine::Create());
   d_->engine_->SetSessionHandler(d_.get());
   d_->engine_->SetOutputHandler(d_.get());
-  d_->engine_->SetUser(buzz::Jid(settings.user(), settings.host(), STR_EMPTY)); 
   if (!settings.resource().empty()) {
     d_->engine_->SetRequestedResource(settings.resource());
   }
-  char pass[settings.pass().GetLength() + 1];
-  settings.pass().CopyTo(pass, true);
-  d_->engine_->SetPass(pass);
   d_->engine_->SetUseTls(settings.use_tls());
 
+  //
+  // The talk.google.com server expects you to use "gmail.com" in the
+  // stream, and expects the domain certificate to be "gmail.com" as well.
+  // For all other servers, we leave the strings empty, which causes
+  // the jid's domain to be used.  "foo@example.com" -> stream to="example.com"
+  // tls certificate for "example.com"
+  //
+  // This is only true when using Gaia auth, so let's say if there's no preauth,
+  // we should use the actual server name
+  if ((settings.server().IPAsString() == buzz::STR_TALK_GOOGLE_COM ||
+      settings.server().IPAsString() == buzz::STR_TALKX_L_GOOGLE_COM) && 
+      pre_auth != NULL) {
+    d_->engine_->SetTlsServer(buzz::STR_GMAIL_COM, buzz::STR_GMAIL_COM);
+  }
+
+  // Set language
+  d_->engine_->SetLanguage(lang);
+
+  d_->engine_->SetUser(buzz::Jid(settings.user(), settings.host(), STR_EMPTY));
 
   d_->pass_ = settings.pass();
   d_->auth_cookie_ = settings.auth_cookie();
   d_->server_ = settings.server();
   d_->proxy_host_ = settings.proxy_host();
   d_->proxy_port_ = settings.proxy_port();
+  d_->allow_plain_ = settings.allow_plain();
   d_->pre_auth_.reset(pre_auth);
 
   return XMPP_RETURN_OK;
@@ -132,12 +149,27 @@ XmppClient::GetState() {
 }
 
 XmppEngine::Error
-XmppClient::GetError() {
+XmppClient::GetError(int *subcode) {
+  if (subcode) {
+    *subcode = 0;
+  }
   if (d_->engine_.get() == NULL)
     return XmppEngine::ERROR_NONE;
-  if (d_->pre_engine_error_ != XmppEngine::ERROR_NONE)
+  if (d_->pre_engine_error_ != XmppEngine::ERROR_NONE) {
+    if (subcode) {
+      *subcode = d_->pre_engine_subcode_;
+    }
     return d_->pre_engine_error_;
-  return d_->engine_->GetError();
+  }
+  return d_->engine_->GetError(subcode);
+}
+
+const XmlElement *
+XmppClient::GetStreamError() {
+  if (d_->engine_.get() == NULL) {
+    return NULL;
+  }
+  return d_->engine_->GetStreamError();
 }
 
 CaptchaChallenge XmppClient::GetCaptchaChallenge() {
@@ -153,7 +185,6 @@ XmppClient::GetAuthCookie() {
   return d_->auth_cookie_;
 }
 
-#if 0
 static void
 ForgetPassword(std::string & to_erase) {
   size_t len = to_erase.size();
@@ -164,7 +195,6 @@ ForgetPassword(std::string & to_erase) {
   // get rid of length
   to_erase.erase();
 }
-#endif
 
 int
 XmppClient::ProcessStart() {
@@ -177,7 +207,7 @@ XmppClient::ProcessStart() {
   }
   else {
     d_->engine_->SetSaslHandler(new PlainSaslHandler(
-              d_->engine_->GetUser(), d_->pass_));
+              d_->engine_->GetUser(), d_->pass_, d_->allow_plain_));
     d_->pass_.Clear(); // done with this;
     return STATE_START_XMPP_LOGIN;
   }
@@ -205,9 +235,11 @@ XmppClient::ProcessCookieLogin() {
     // maybe split out a case when gaia is down?
     if (d_->pre_auth_->HadError()) {
       d_->pre_engine_error_ = XmppEngine::ERROR_AUTH;
+      d_->pre_engine_subcode_ = d_->pre_auth_->GetError();
     }
     else {
       d_->pre_engine_error_ = XmppEngine::ERROR_UNAUTHORIZED;
+      d_->pre_engine_subcode_ = 0;
       d_->captcha_challenge_ = d_->pre_auth_->GetCaptchaChallenge();
     }
     d_->pre_auth_.reset(NULL); // done with this
@@ -220,7 +252,6 @@ XmppClient::ProcessCookieLogin() {
 
   // transfer ownership of pre_auth_ to engine
   d_->engine_->SetSaslHandler(d_->pre_auth_.release());
-  
   return STATE_START_XMPP_LOGIN;
 }
 
@@ -252,12 +283,17 @@ XmppClient::Disconnect() {
   return XMPP_RETURN_OK;
 }
 
-XmppClient::XmppClient(Task * parent) : Task(parent), 
-    delivering_signal_(false) {
+XmppClient::XmppClient(Task * parent) 
+    : Task(parent), 
+      delivering_signal_(false),
+      valid_(false) {
   d_.reset(new Private(this));
+  valid_ = true;
 }
 
-XmppClient::~XmppClient() {}
+XmppClient::~XmppClient() {
+  valid_ = false;
+}
 
 const Jid &
 XmppClient::jid() {
@@ -318,7 +354,8 @@ XmppClient::Private::OnSocketRead() {
 
 void
 XmppClient::Private::OnSocketClosed() {
-  engine_->ConnectionClosed();
+  int code = socket_->GetError();  
+  engine_->ConnectionClosed(code);
 }
 
 void
